
// ========================================
// The basic form to create plugins.
//
// (C) Copyright 1997-1998 Andrei N. Kashin
// E-mail: Andrei.Kashin@Fysik.uu.se
//
//   Do not remove this CopyRight.
// =======================================

unit mform1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  npapi,StdCtrls;

type

TPoints = array [0..100] of TPoint;

TForm1 = class(TPluginForm)
    ScrollBar1: TScrollBar;
    NetscapeStream1: TNetscapeStream;
    Label1: TLabel;
    ScrollBar2: TScrollBar;
    Label2: TLabel;
    ScrollBar3: TScrollBar;
    Label3: TLabel;
    Label4: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure ScrollBar1Change(Sender: TObject);
    procedure ScrollBar2Change(Sender: TObject);
    procedure ScrollBar3Change(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure NetscapeStream1AsFile(Sender: TObject; FileName: PChar);
    procedure NetscapeStream1CloseStream(Sender: TObject;
      Reason: Smallint);
    procedure NetscapeStream1Read(Sender: TObject; Buf: Pointer;
      ByteCount: Integer);
   public
    Points:TPoints;
    NPoints:integer;
    MStream:TMemoryStream;
    BytesRead:integer;
    procedure MRedraw;
    procedure InitFigure;
    function GetScrollPos(ArgName:PChar):integer;
    procedure ReadFromMemory;
    procedure ReadPolygon(var f:text);
   protected
    procedure NM_NewStreamHandler(var Msg:TMessage); message NM_NewStream;
 end;


implementation

{$R *.DFM}

const RectanglePoints:array [0..3] of TPoint =
 ((X:0;Y:0),(X:200;Y:0),(X:200;Y:200),(X:0;Y:200));

procedure TForm1.NM_NewStreamHandler(var Msg:TMessage);
begin
 NetscapeStream1.Accept(PNPStream(Msg.LParam),InstanceID);
 {don't modify this call. If you doesn't support streams at all,
  simply remove or comment it.}

 {you can add your initializations below}
 MStream:=TMemoryStream.Create();
 BytesRead:=0;

 inherited;  // must be last
end;

function MakeColor(R,G,B:byte):TColor;
begin
 Result:=integer(R) or integer(G) shl 8 or integer(B) shl 16;
end;

procedure TForm1.MRedraw;
begin
 if NPoints>2 then begin
  Canvas.Brush.Color:=MakeColor(
   ScrollBar1.Position,ScrollBar2.Position,ScrollBar3.Position);
  Canvas.Polygon(Slice(Points,NPoints));
 end;
end;

procedure TForm1.InitFigure;
begin
 NPoints:=4;
 Move(RectanglePoints,Points,SizeOf(TPoint)*4);
end;


{This function demonstrates how to use NPN_GetArgValue
 function to retrive parameters from EMBED tag.
 It's used to set initial values of scrollbars' positions }
function TForm1.GetScrollPos(ArgName:Pchar):integer;
var s:string;
    i,code:integer;
begin
 s:=StrPas(NPN_GetArgValue(InstanceID,ArgName));
 if s='' then Result:=0
 else begin
   val(s,i,code);
   if (code>0) or (i<0) or (i>255) then Result:=0
   else Result:=i;
 end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 InitFigure;
 ScrollBar1.Position:=GetScrollPos('Red');
 ScrollBar2.Position:=GetScrollPos('Green');
 ScrollBar3.Position:=GetScrollPos('Blue');
 MRedraw;
end;

procedure TForm1.ScrollBar1Change(Sender: TObject);
begin
 MRedraw;
end;

procedure TForm1.ScrollBar2Change(Sender: TObject);
begin
 Mredraw;
end;

procedure TForm1.ScrollBar3Change(Sender: TObject);
begin
 MRedraw;
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
 MRedraw;
end;

procedure TForm1.ReadPolygon(var f:TextFile);
begin
  NPoints:=0;
  while not EOF(f) and (NPoints<=100) do begin
   ReadLn(f,Points[NPoints].X,Points[Npoints].Y);
   inc(NPoints);
  end;
end;

function DevCommon(var F:TTextRec):integer;
begin
 Result:=0;
end;

procedure TForm1.ReadFromMemory;
var f:TextFile;
    EofChar:char;
begin
 if MStream=nil then exit;
 EofChar:=#$1A;
 MStream.Write(EofChar,1);
 with TTextRec(f) do begin
  Mode:=fmInput;
  BufSize:=MStream.Size;
  BufEnd:=BufSize;
  BufPtr:=MStream.Memory;
  BufPos:=0;
  FlushFunc:=@DevCommon;
 end;
 try
  ReadPolygon(f);
 except
  InitFigure;
 end;
 Repaint;
end;

{This procedure reads polygon's coordinates from a file.
 It is called when URL has been copied on local computer
 and ready to open as usual file. Note that TNetscapeStream
 StreamType property must be stAsFile. You'll never get
 onAsFile event for stNormal stream type.}
procedure TForm1.NetscapeStream1AsFile(Sender: TObject; FileName: PChar);
var f:TextFile;
begin
 try
  AssignFile(f,StrPas(FileName));
  Reset(f);
  ReadPolygon(f);
  CloseFile(f);
 except
  InitFigure;
  CloseFile(f);
 end;
 Repaint;
 Label4.Caption:='File received: '+StrPas(FileName);
end;

procedure TForm1.NetscapeStream1CloseStream(Sender: TObject;
  Reason: Smallint);
var s:string;
begin
  s:='';
  case Reason of
   NPRES_DONE:  begin   s:='Transfer complete.';
                        ReadFromMemory;
                end;
   NPRES_USER_BREAK:  s:='User has pressed a STOP button.';
   NPRES_NETWORK_ERR: s:='A network error has occured.';
  end;
  Label4.Caption:=s;
  MStream.Free;
  MStream:=nil;
end;

{When StreamType property of TnetscapeStream is stNormal you'll get
 succesive onRead events for each portion of data transfered.}
procedure TForm1.NetscapeStream1Read(Sender: TObject; Buf: Pointer;
  ByteCount: Integer);
var s:string;
begin
 if MStream<>nil then
 MStream.Write(Buf^,ByteCount);
 Inc(BytesRead,ByteCount);
 Str(BytesRead,S);
 Label4.Caption:='Bytes received: ' + s;
end;

initialization
  RegisterPlugInForm(TForm1); // to register our form
  {ConfirmThreadSafe;}
end.
